#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <stdint.h>

//declare button
typedef uint8_t Pin;
Pin button = 2;
int what_scale_to_use = 1;

//initialize lcd
LiquidCrystal_I2C lcd(39,16,2);

const int trigger = 5;
const int echo = 4;

const int piezo = 10;

int distance = 0;
int distanceHigh = 0;

int lengthOfScale = 0;

int note = 0;

//A Minor pentatonic scale
int scale[] = {
  147, 165, 196, 220, 262, 294, 330, 392, 440,
  523, 587, 659, 784, 880, 1047, 1175, 1319, 1568,
  1760, 2093, 2349
};

//A minor pentatonic note representation
String notes[] = {
  "D#", "E", "G", "A", "C", "D", "E", "G", "A", "C", "D", "E", "G", "A", "C", "D", "E", "G", "A", "C", "D"
};

//C Major scale
int scale2[] = {
 131, 147, 165, 175, 196, 220, 247, 262, 294,
 330, 349, 392, 440, 494, 523, 587, 659, 698,
 784, 880, 988, 1047
};

// C Major scal note representation
String notes2[] = {
  "B", "D", "E", "F", "G", "A", "B", "C", "D", "E", "F", "G", "A", "B", "C", "D", "E", "F", "G", "A", "B", "C"
};


void change_scale() {
  
  if(what_scale_to_use == 1) {

    what_scale_to_use = 2;
  } else {

    what_scale_to_use = 1;
  }
}

void setup() {
  Serial.begin(9600);

  pinMode(button, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(button), change_scale, FALLING);
  
  pinMode(trigger, OUTPUT);
  pinMode(echo, INPUT);

  while (millis() < 5000) {
    digitalWrite(trigger, HIGH);
    digitalWrite(trigger, LOW);
    distance = pulseIn(echo, HIGH);

    if (distance > distanceHigh) {
      distanceHigh = distance;
    }
  }

  for (byte i = 0; i < (sizeof(scale) / sizeof(scale[0])); i++) {
    lengthOfScale += 1;
  }
  
  lcd.init();
  lcd.clear();         
  lcd.backlight();

}

void loop() {

  digitalWrite(trigger, HIGH);
  digitalWrite(trigger, LOW);
  
  distance = pulseIn(echo, HIGH);

  if(what_scale_to_use == 1) {

    note = map(distance, 250, distanceHigh, scale[0], scale[lengthOfScale - 1]);

    for (byte j = 0; j < (lengthOfScale); j++) {

      if (note == scale[j]) {
        noTone(piezo);
        delay(30);
        tone(piezo, note);
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("A Minor, Note:");
        lcd.setCursor(0, 1);
        lcd.print(notes[j]);
        delay(30);
        break;
      }
      else if (note > scale[j] && note < scale[j + 1]) {
        note = scale[j];
        noTone(piezo);
        delay(30);
        tone(piezo, note);
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("A Minor, Note:");
        lcd.setCursor(0, 1);
        lcd.print(notes[j]);
        delay(30);
        break;
      }
    }
  } else {

    note = map(distance, 250, distanceHigh, scale2[0], scale2[lengthOfScale - 1]);

    for (byte j = 0; j < (lengthOfScale); j++) {

      if (note == scale2[j]) {
        noTone(piezo);
        delay(30);
        tone(piezo, note);
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("C Major, Note:");
        lcd.setCursor(0, 1);
        lcd.print(notes2[j]);
        delay(30);
        break;
      }
      else if (note > scale2[j] && note < scale2[j + 1]) {
        note = scale2[j];
        noTone(piezo);
        delay(30);
        tone(piezo, note);
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("C Major, Note:");
        lcd.setCursor(0, 1);
        lcd.print(notes2[j]);
        delay(30);
        break;
      }
    }
  }
  

  delay(30);
}
